import React from 'react';
import * as ReactDOM from 'react-dom';
import { PortalManager } from '@highlight-ui/utils-portal-manager';
import { Button } from '@highlight-ui/button';
import { Icon } from '@highlight-ui/icon';
import { Typography } from '@highlight-ui/typography';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import classnames from 'classnames';
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

/* global Reflect, Promise */

var extendStatics = function (d, b) {
  extendStatics = Object.setPrototypeOf || {
    __proto__: []
  } instanceof Array && function (d, b) {
    d.__proto__ = b;
  } || function (d, b) {
    for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p];
  };

  return extendStatics(d, b);
};

function __extends(d, b) {
  if (typeof b !== "function" && b !== null) throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
  extendStatics(d, b);

  function __() {
    this.constructor = d;
  }

  d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}

function createTimer(callback, delay) {
  var timerId = 0;
  var start = new Date(0);
  var remaining = delay;
  var timer = {
    reset: function () {
      clearTimeout(timerId);
      timerId = 0;
    },
    pause: function () {
      window.clearTimeout(timerId);
      remaining -= +new Date() - +start;
    },
    resume: function () {
      start = new Date();
      window.clearTimeout(timerId);
      timerId = window.setTimeout(callback, remaining);
    }
  };
  timer.resume();
  return timer;
}

var variantToIconNameMap = {
  highlight: 'info-circle',
  success: 'check-circle',
  warning: 'exclamation-triangle',
  critical: 'exclamation-circle'
};

var variantToIconName = function (variant) {
  return variantToIconNameMap[variant];
};

var styles$1 = {
  "container": "Toast-module__sbo_kiyp__v5-1-17",
  "inner": "Toast-module__3yxPby4t__v5-1-17",
  "open-animation": "Toast-module__33sqCFfu__v5-1-17",
  "close-animation": "Toast-module__w_RTVO85__v5-1-17",
  "progressBarInner": "Toast-module__351uTXcg__v5-1-17",
  "contentContainer": "Toast-module__3W3ZrAqN__v5-1-17",
  "bodyContainer": "Toast-module__1cZEAN9K__v5-1-17",
  "iconContainer": "Toast-module__2njM2QSH__v5-1-17",
  "closeButtonContainer": "Toast-module__-fbljNAe__v5-1-17",
  "closeButton": "Toast-module__yEZbH-JS__v5-1-17",
  "footerContainer": "Toast-module__2g9SAgaN__v5-1-17",
  "progressBarContainer": "Toast-module__YLOA9JSa__v5-1-17",
  "progress-bar": "Toast-module__zzrFIkb4__v5-1-17",
  "highlight": "Toast-module__uJzZ5GMJ__v5-1-17",
  "success": "Toast-module__37lkfJHP__v5-1-17",
  "warning": "Toast-module__nHjCLFB7__v5-1-17",
  "critical": "Toast-module__1E7MHWSV__v5-1-17",
  "actionsContainer": "Toast-module__3XnNpf86__v5-1-17"
};

var themeToVariant = function (theme) {
  switch (theme) {
    case 'danger':
      return 'critical';

    case 'success':
      return 'success';

    case 'warning':
      return 'warning';

    case 'info':
    default:
      return 'highlight';
  }
};

var LEAVING_ANIMATION_DURATION = 200;

var Toast =
/** @class */
function (_super) {
  __extends(Toast, _super);

  function Toast(props) {
    var _this = _super.call(this, props) || this;

    _this.closeTimer = null;
    _this.removeTimer = null;

    _this.close = function () {
      var onRequestToClose = _this.props.onRequestToClose;

      _this.clearCloseTimer();

      if (typeof onRequestToClose === 'function') {
        onRequestToClose();
      }
    };

    _this.startCloseTimer = function () {
      var _a = _this.props,
          autoClose = _a.autoClose,
          duration = _a.duration;

      if (autoClose && duration) {
        _this.closeTimer = createTimer(_this.close, duration);
      }
    };

    _this.clearCloseTimer = function () {
      if (_this.removeTimer) {
        clearTimeout(_this.removeTimer);
      }

      if (_this.closeTimer) {
        _this.closeTimer.reset();
      }
    };

    _this.handleMouseEnter = function () {
      if (_this.closeTimer) {
        _this.closeTimer.pause();
      }
    };

    _this.handleMouseLeave = function () {
      if (_this.closeTimer) {
        _this.closeTimer.resume();
      }
    };

    _this.state = {
      isVisible: true
    };
    return _this;
  }

  Toast.prototype.componentDidMount = function () {
    var onOpen = this.props.onOpen;

    if (onOpen) {
      onOpen();
    }

    this.startCloseTimer();
  };

  Toast.prototype.componentDidUpdate = function (prevProps) {
    var _a = this.props,
        isVisible = _a.isVisible,
        onRequestToRemove = _a.onRequestToRemove;

    if (prevProps.isVisible !== isVisible) {
      // eslint-disable-next-line react/no-did-update-set-state
      this.setState({
        isVisible: isVisible
      });

      if (!isVisible && typeof onRequestToRemove === 'function') {
        this.removeTimer = setTimeout(onRequestToRemove, LEAVING_ANIMATION_DURATION);
      }
    }
  };

  Toast.prototype.componentWillUnmount = function () {
    var onClose = this.props.onClose;

    if (onClose) {
      onClose();
    }

    this.clearCloseTimer();
  };

  Toast.prototype.render = function () {
    var state = this.state;
    var _a = this.props,
        actions = _a.actions,
        autoClose = _a.autoClose,
        className = _a.className,
        children = _a.children,
        duration = _a.duration,
        metadata = _a.metadata,
        onRequestToClose = _a.onRequestToClose,
        showCloseButton = _a.showCloseButton,
        theme = _a.theme,
        _b = _a.variant,
        variant = _b === void 0 ? 'highlight' : _b;
    var variantOrDeprecatedTheme = typeof theme !== 'undefined' ? themeToVariant(theme) : variant;
    var progressBarInner = classnames(styles$1.progressBarInner, styles$1[variantOrDeprecatedTheme]);
    var iconName = variantToIconName(variantOrDeprecatedTheme);
    var iconColor = "icon-" + variantOrDeprecatedTheme;
    return /*#__PURE__*/React.createElement("div", __assign({
      className: classnames(className, styles$1.container),
      role: "log"
    }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", {
      "data-state": state.isVisible ? 'entering' : 'leaving',
      className: styles$1.inner,
      onMouseEnter: this.handleMouseEnter,
      onMouseLeave: this.handleMouseLeave
    }, /*#__PURE__*/React.createElement("div", {
      className: styles$1.contentContainer
    }, /*#__PURE__*/React.createElement("div", {
      className: styles$1.iconContainer
    }, /*#__PURE__*/React.createElement(Icon, {
      name: iconName,
      color: iconColor,
      size: 20,
      type: "solid",
      metadata: suffixMetadata(metadata, 'icon')
    })), children && /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
      className: styles$1.bodyContainer
    }, /*#__PURE__*/React.createElement(Typography, {
      token: "body-base",
      color: "text-default"
    }, children)), actions && /*#__PURE__*/React.createElement("div", __assign({
      className: styles$1.actionsContainer
    }, parseMetadata(suffixMetadata(metadata, 'actions-container'))), actions)), showCloseButton && /*#__PURE__*/React.createElement("div", {
      className: styles$1.closeButtonContainer
    }, /*#__PURE__*/React.createElement(Button, __assign({
      type: "button",
      className: styles$1.closeButton,
      tabIndex: -1,
      onClick: onRequestToClose
    }, parseMetadata(suffixMetadata(metadata, 'close-btn'))), /*#__PURE__*/React.createElement(Icon, {
      name: "times",
      color: "icon-on-secondary"
    })))), autoClose && duration && /*#__PURE__*/React.createElement("div", __assign({
      className: styles$1.progressBarContainer,
      role: "progressbar"
    }, parseMetadata(suffixMetadata(metadata, 'progress-bar'))), /*#__PURE__*/React.createElement("div", {
      className: progressBarInner,
      style: {
        animationDuration: duration / 1000 + "s"
      }
    }))));
  }; // eslint-disable-next-line react/static-property-placement


  Toast.defaultProps = {
    variant: 'highlight',
    autoClose: true,
    duration: 5000,
    showCloseButton: true,
    actions: undefined,
    onOpen: undefined,
    onClose: undefined
  };
  return Toast;
}(React.PureComponent);

var styles = {
  "container": "ToastManager-module__1JSUhSHH__v5-1-17"
};

var hasCustomId = function (settings) {
  return Object.keys(settings).includes('id');
};

var ToastManager =
/** @class */
function (_super) {
  __extends(ToastManager, _super);

  function ToastManager(props) {
    var _this = _super.call(this, props) || this;

    _this.getToastHandlers = function () {
      var toasts = _this.state.toasts;
      return toasts;
    };

    _this.closeAll = function () {
      var toasts = _this.getToastHandlers();

      toasts.forEach(function (toast) {
        if (toast && typeof toast.onRequestToClose === 'function') {
          toast.onRequestToClose();
        }
      });
    };

    _this.notify = function (options) {
      // If there's a custom toast ID passed, close existing toasts with the same custom ID
      if (hasCustomId(options)) {
        var toasts = _this.state.toasts;
        toasts.forEach(function (toast) {
          // Since unique ID is still appended to a custom ID, skip the unique ID and check only prefix
          if (toast.id && String(toast.id).startsWith(options.id || '')) {
            _this.closeToast(toast.id);
          }
        });
      }

      var toast = _this.createToastInstance(options);

      _this.setState(function (previousState) {
        return {
          toasts: __spreadArray([toast], previousState.toasts, true)
        };
      });

      return toast;
    };

    _this.createToastInstance = function (options) {
      if (options === void 0) {
        options = {
          children: null
        };
      }

      ToastManager.idCounter += 1;
      var uniqueId = ToastManager.idCounter;
      var id = hasCustomId(options) ? (options.id || '') + "-" + uniqueId : String(uniqueId);
      return __assign(__assign({}, options), {
        id: id,
        onRequestToClose: function () {
          return _this.closeToast(id);
        },
        onRequestToRemove: function () {
          return _this.removeToast(id);
        }
      });
    };

    _this.closeToast = function (id) {
      _this.setState(function (previousState) {
        return {
          toasts: previousState.toasts.map(function (toast) {
            if (toast.id === id) return __assign(__assign({}, toast), {
              isVisible: false
            });
            return toast;
          })
        };
      });
    };

    _this.removeToast = function (id) {
      _this.setState(function (previousState) {
        return {
          toasts: previousState.toasts.filter(function (toast) {
            return toast.id !== id;
          })
        };
      });
    };

    props.bindNotify(_this.notify); // props.bindGetToasts(this.getToasts); // TODO: Investigate why getToasts() never worked

    props.bindCloseAll(_this.closeAll);
    _this.state = {
      toasts: []
    };
    return _this;
  }

  ToastManager.prototype.render = function () {
    var toasts = this.state.toasts;
    return /*#__PURE__*/React.createElement("div", {
      className: styles.container
    }, toasts.map(function (props) {
      return /*#__PURE__*/React.createElement(Toast, __assign({
        key: props.id
      }, props));
    }));
  };

  ToastManager.idCounter = 0;
  return ToastManager;
}(React.PureComponent);

var Toaster =
/** @class */
function () {
  function Toaster() {
    var _this = this;

    this.closeAllHandler = function () {};

    this.getToastsHandler = function () {
      return [];
    };

    this.notifyHandler = function () {};

    this.bindNotify = function (handler) {
      _this.notifyHandler = handler;
    };

    this.bindGetToasts = function (handler) {
      _this.getToastsHandler = handler;
    };

    this.bindCloseAll = function (handler) {
      _this.closeAllHandler = handler;
    };

    this.closeAll = function () {
      _this.closeAllHandler();
    };

    this.notify = function (options) {
      _this.notifyHandler(options);
    };

    var container = document.createElement('div');
    container.setAttribute('personio-toaster-container', '');
    PortalManager.containerInstance().appendChild(container);
    ReactDOM.render( /*#__PURE__*/React.createElement(ToastManager, {
      bindNotify: this.bindNotify,
      bindGetToasts: this.bindGetToasts,
      bindCloseAll: this.bindCloseAll
    }), container);
    return this;
  }

  Object.defineProperty(Toaster.prototype, "toasts", {
    get: function () {
      return this.getToastsHandler();
    },
    enumerable: false,
    configurable: true
  });
  return Toaster;
}();

var Toaster$1 = new Toaster();
export { Toaster$1 as toaster };
//# sourceMappingURL=index.js.map
